function [x_all,xf_all,u_all,active_sensing_all,P_all] = run_biswas(xx,n,model,params,refuge)

counter = 0;                                                % counter for unstable simulations
sim_L = length(refuge.pos);                                 % similation length

% optimum parameters
params.m_noise = xx(1);
params.scene(1) = xx(2);
params.scene(2) = xx(3);
params.r_noise = xx(4);
params.act_params = xx(5:7);

ekf_param.F = model.A;                                      % state transition matrix
ekf_param.H = @(x)[0 0 params.scene(1)*x(4) ... `           % observation matrix
    params.scene(1)*x(3)+params.scene(2)];
ekf_param.P = diag([0.01 0.01 0.001 0.001]);                % initial covariance
ekf_param.Q = diag([params.p_noise params.r_noise]);        % process noise
ekf_param.R = diag(params.m_noise);                         % observation noise
ekf_param.L = model.B;                                      % matrix that process noise goes through  

state_func = @(x,u) model.A*x + model.B*[u(1);u(2)];
meas_func = @(x,noise) params.scene(1)*x(3)*x(4)+ params.scene(2)*x(4) + noise;

% initialize variables
x_all = nan(n,4,sim_L);
xf_all = nan(n,4,sim_L);
u_all = nan(n,sim_L);
active_sensing_all = nan(n,sim_L);
P_all = nan(n,4,4,sim_L);
x_init = nan(4,1);

j = 1;
while j <= n
    
    unstable = 0; % flag for unstable simulations
    
    % choose a random initial condition
    x_init(1:2,1) = [refuge.pos(1)+randn*params.init_state_prob(2)+params.init_state_prob(1);refuge.vel(1)+randn*params.init_state_prob(4)+params.init_state_prob(3)];
    x_init(3:4,1) = [refuge.pos(1) - x_init(1,1); refuge.vel(1) - x_init(2,1)];
    
    % initialize covariance matrix of EKF
    ekf_param.P = diag([1 0.01 0.001 0.001]);
    P = nan(4,4,sim_L);
    P(:,:,1) = ekf_param.P;
    
    % initialize variables
    active_sensing = zeros(1,sim_L);
    u = nan(1,sim_L);  
    y = nan(1,sim_L);
    
    % initialize state
    x = nan(4,sim_L);
    x(:,1) = x_init;
    % initialize state estimation
    xf = x;
    xf(:,1) = x_init+[sqrt(ekf_param.P(1,1)) sqrt(ekf_param.P(2,2)) sqrt(ekf_param.P(3,3)) sqrt(ekf_param.P(4,4))]';

    flg = nan(1,sim_L);
    flg(1) = 0;
    
    for i = 1:sim_L-1

        % count unstable simulations if it is more than 5 percent give error
        if sum(abs(xf(:,i))>50)
            counter = counter + 1;
            unstable = 1;
            if counter>0.05*n
                error('more than 5 percent is not stable')
            end
            break;
        end   

        % active sensing generation
        if params.act_params(2)+params.act_params(3) < (ekf_param.P(3,3)+ekf_param.P(4,4)) || flg(i) == 1
            
            active_sensing(i+1) = params.act_params(1)*randn(1,1);
            flg(i+1) = 1;
            if params.act_params(2) > (ekf_param.P(3,3)+ekf_param.P(4,4))
                flg(i+1) = 0;
            end
        else
            active_sensing(i+1) = 0;
            flg(i+1) = 0;                     
        end
  
        % state feedback and active sensing
        u(:,i) = params.Kp*xf(3,i) + params.Kd*xf(4,i)+active_sensing(i+1);
        % saturate the input if it is high
        u(:,i) = sign(u(:,i))*min(3,abs(u(:,i)));
        
        % discrete-time transition
        x(3:4,i) = [refuge.pos(i) - x(1,i); refuge.vel(i) - x(2,i)]; 
        x(:,i+1) = state_func(x(:,i),[u(:,i)+normrnd(0,sqrt(params.p_noise));refuge.acc(i)]);
        % take measurement 
        y(:,i+1) = meas_func(x(:,i+1), normrnd(0,sqrt(params.m_noise))');
        
        % run EKF
        [xf(:,i+1),ekf_param] = ekf(xf(:,i), [u(:,i);0], state_func, meas_func, y(:,i+1), ekf_param);
        P(:,:,i+1) = ekf_param.P;
    end
    
    if unstable == 1
        continue;
    end
   
    % record one instatanetion
    x_all(j,:,:) = x;
    xf_all(j,:,:) = xf;
    u_all(j,:) = u;
    active_sensing_all(j,:) = active_sensing;
    P_all(j,:,:,:) = P;

    j = j + 1;
end

% get rid of first 20 seconds
x_all(:,:,1:500) = [];
xf_all(:,:,1:500) = [];
u_all(:,1:500) = [];
active_sensing_all(:,1:500) = [];
P_all(:,:,:,1:500) = [];

